<?php
/**
 * Implementation for the main application controller plugin class.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Front controller plugin for cookies and other application things.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class MainAppPlugin extends Zend_Controller_Plugin_Abstract
{
//==============================================================================
/**
 * Plugin hook before front controller begins evaluating the request.
 *
 * Sets language and locale if necessary parameters are presented in the url.
 * This is a hack, because directly modifies the $request->pathInfo, which
 * is the used for matching routes.
 */
public function routeStartup(Zend_Controller_Request_Abstract $request)
{
 //set host part for the base application URL
 AppBaseUrl::setHost($request->getServer("SERVER_NAME"));

 $app=HomeWebApp::getInstance();

 //pathinfo = requestUri - baseUrl
 //e.g.:
 //whole URL = http://server_name/homepage/blog/myitem
 //requestUri = /homepage/blog/myitem, baseUrl= /homepage, pathInfo=/blog/myitem
 $pathInfo=$request->getPathInfo();

 //check if lang identfier is in the pathinfo
 if (preg_match('#^/([a-z]{2}/|[a-z]{2}$)#',$pathInfo,$matches)) {
  if (HomeWebApp::DEBUG) $app->getLog()->info("lang matched in pathInfo");
  $urlLang=rtrim($matches[1],'/');
  //remove lang identifiers from the pathInfo and sets as new pathinfo
  $pathInfo = strlen($pathInfo)==3 ? "/" : substr($pathInfo,3);
  $request->setPathInfo($pathInfo);
 } else {
  //language isn't in the url, default lang "without url specification" is cs
  $urlLang="cs";
 }

 if (HomeWebApp::DEBUG) $app->getLog()->info("urlLang: $urlLang");
 if (HomeWebApp::DEBUG) $app->getLog()->info("path info after language cut:".$request->getPathInfo());

 //set application language
 $app->setLang($urlLang);
 //set language in all urls
 if ($urlLang!="cs") AppUrlEnum::getInstance()->setLang($urlLang);
 
 //set language in viewrenderer views
 $viewRenderer=Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
 //set language
 $viewRenderer->view->currentLang=$app->getLocale()->getLanguage();
}

//==============================================================================
/**
 * Plugin hook after Zend_Controller_Front exits a whole dispatch loop.
 *
 * Add cookies and sets some headers, encoding
 */
public function dispatchLoopShutdown()
{
 $request  = $this->getRequest();
 $response = $this->getResponse();

 //checks for some requests flags, mostly for ajax response, because
 //all ajax html in the response must be in the utf-8
 if ($request->getParam('encodeUTF8')) {
  $response->setBody(iconv("iso-8859-2","utf-8",$response->getBody()));
 }
 $response->setHeader("Cache-Control","max-age=7200,must-revalidate",true);
}

#END CLASS
}
