<?php
/**
 * Implementation of blog Index controller actions
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Serve blog index actions
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class Blog_IndexController extends Zend_Controller_Action
{
 /** cookie variable name for the serialized comment form
  * data
  */  
 const COMMENTCOOKIE = "commentInfo";
 
//==============================================================================
/**
 * Adds some paths for view scripts and setups right block on the page
 */   
public function init()
{
 $config = Zend_Registry::get("conf");

 $this->view->addHelperPath($config->defaultViewDir.DIRECTORY_SEPARATOR."helpers");
 $this->view->addScriptPath($config->defaultViewDir.DIRECTORY_SEPARATOR."scripts");
}

//==============================================================================
public function righBlock()
{
 //right block
 $category=new Category();
 $this->view->allCategories=$category->getAll(null,null,array("article"));

 //Renders rightblock into 'rightBlock' segment
 $response = $this->getResponse();
 $response->insert('rightBlock', $this->view->render('blogMenu.phtml'));

 //unset, because $this->view is used in other controller actions and must be
 //without any variables in these actions
 unset($this->view->allCategories);

 //workaround for the bug?? in the Zend_View_Helper_Action
 //this is necessary because blogMenu.phtml calls action view helper, which
 //calls the viewRenderer and its state (_noRender member variable) is changed
 //in this calls
 $this->_helper->viewRenderer->setNoRender(false);
}

//==============================================================================
/**
 * Action handler for index page.
 */
public function indexAction()
{
 //right block
 $this->righBlock();

 if ($this->_hasParam("entry")) {
  $this->_forward("view-entry");
  return;
 }

 $article = new Article();
 
 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;

 //page navigator
 $pgNav=new PageNav($pageNo,$article->getCount(),Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=AppUrlEnum::getInstance()->blogIndex;

 $this->view->articles=$article->getAll($pageNo,$pgNav->getPerPageItems(),
   array("comment","author")
 );

 //page navigator
 $this->view->pageNav=$pgNav;
 
 //page title
 $this->view->pageTitle="Blog";
}

//==============================================================================
/**
 * Action handler for one blog entry.
 * Serves add action for comments and view comments for the current article also.   
 */
public function viewEntryAction()
{
 $articleTitleUrl=$this->_getParam("entry");
 $currentArticle = new Article();
 
 $found=$currentArticle->findByTitleUrl($articleTitleUrl);
 //false - article not found
 if (!$found) {
  if (HomeWebApp::DEBUG) {
   echo "article doesn't exist, in production, 404 is invoked";
   exit;
  } else { 
   $this->_forward("error","Error",null,
    array("error_handler"=>Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_ACTION));
   return;
  } 
 }

 //right block
 $this->righBlock();

 $this->view->article=$currentArticle;
 $this->view->author=$currentArticle->getAuthor();
 $this->view->comments=$currentArticle->getComments();
 $this->view->trackbacks=$currentArticle->getTrackBacks();
 $this->view->ratingInfo=$currentArticle->getRatingInfo();
 $this->view->trackbackRDF=$this->trackbackArticleRDF($currentArticle);

 //next and previous articles
 $this->view->nextArticle=$currentArticle->getNext();
 $this->view->previousArticle=$currentArticle->getPrevious();

 //check if already voted
 $articleRating=$currentArticle->getRating();
 $articleRating->setIpAddr($this->getRequest()->REMOTE_ADDR);
 if ($this->getRequest()->has($articleRating->getCookieName())
  || $articleRating->wasVoted() )
 {
  $this->view->voted=true;
 } else {
  $this->view->voted=false;
 }

 //page title
 $this->view->pageTitle=$currentArticle->getTitle();

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogEntryName($currentArticle->getTitleUrl());
 $urlEnum->setBlogEntryId($currentArticle->getId());
 $currentEntryUrl=$urlEnum->blogEntry;

 $this->view->trackBackUrl=$urlEnum->trackBackEntry;

 //just alias for $this->_helper->flashMessenger
 $messanger=$this->_helper->getHelper('FlashMessenger');

 //vaditate and save comment if it's submitted via form
 if ($this->_hasParam("formAction") && $this->_hasParam("formAction")=="addCommentDb") {
  $data=$this->getRequest()->getPost();

  $filters = array(
    'username' =>'StringTrim',
    'mainEmail' => 'StringTrim',
    'message' => 'StringTrim',
    'www' => array('StringTrim')
   );

  $validators = array(
    'username'   => 'NotEmpty',
    'mainEmail'   => 'EmailAddress',
    'message' => 'NotEmpty',
    'www' => array(new UrlValid(),'allowEmpty' => true),
    'homepage' => array(
        'Alnum',
        'presence' => 'required'
     ) 
  );

  $inputFilter = new Zend_Filter_Input($filters, $validators);
  $inputFilter->setData($data);

  //filter is valid, we can save comment
  if ($inputFilter->isValid()) {
   //check spam phrase
   if ($inputFilter->homepage!="NejsemSpammer") {
    $messanger->addMessage("Spam not allowed");
    $this->_helper->redirector->setGotoUrl($currentEntryUrl,array("prependBase"=>false));
    $this->_helper->redirector->redirectAndExit();
    return;
   }

   $username=htmlspecialchars($inputFilter->getUnescaped("username"));
   $email=$inputFilter->getUnescaped('mainEmail');
   $commentMessage=$inputFilter->getUnescaped('message');
   $commentMessage=Comment::prepareCommentText($commentMessage);

   $www=$inputFilter->getUnescaped('www');
   if (!empty($www) && !preg_match("|^http(s)?://|i",$www)) $www="http://".$www;
     
   try {
    $comment=new Comment();
    $comment->setArticleId($currentArticle->getId());
    $comment->setMessage($commentMessage);
    $comment->setAuthorName($username);
    $comment->setAuthorEmail($email);
    $comment->setAuthorWWW($www);
    
    //insert new item into the data source
    $comment->insert();
    
    //sets cookie with comment dialog info, will be used as source for the
    //comment form predefined variables
    self::setCommentCookie($username,$email,$www);
   } catch (Zend_Db_Exception $e) {
    $messages[]="Sorry, cannot save record comment, database error was occured"
     . (HomeWebApp::DEBUG ? ":".$e->getMessage() : ".");
    $this->view->messages=$messages;
    //remove formAction parameter from the $_POST, it's impossible unset superglobal via zend request class
    unset($_POST["formAction"]);
    $this->_forward("view-entry");
    return;
   }
   //some OK message, actually not necessary
  } else {
   //invalid data
   $messages[]="Data are invalid:";
   $messages=array_merge($messages,Utils::getMessagesOneArray($inputFilter));
   $this->view->messages=$messages;
   //remove formAction parameter from the $_POST, it's impossible unset superglobal
   //via zend request class
   unset($_POST["formAction"]);
   $this->_forward("view-entry");
   return;
  }

  $this->_helper->redirector->setGotoUrl($currentEntryUrl,array("prependBase"=>false));
  $this->_helper->redirector->redirectAndExit();

  //end if ($this->_hasParam("formAction") ...
 } else {
  //form wasn't submited it means normal article view, we can add pageview
  //for the article
  $currentArticle->incViewCount(true);
 }


 if ($messanger->hasMessages()) {
  $this->view->messages=$messanger->getMessages();
 }

 //old (and in most cases invalid) send data
 if ($this->_hasParam("username")) {
  $this->view->formData=$this->getRequest()->getPost();
 } elseif ($this->getRequest()->has(self::COMMENTCOOKIE)) {
  $this->view->formData=unserialize($this->getRequest()->getCookie(self::COMMENTCOOKIE));
 }

}

//==============================================================================
/**
 * Action hadler for a view of articles category.
 */
public function viewCategoryAction()
{
 $category=new Category();

 $found=$category->findByTitleUrl($this->_getParam("name"));
 //false - category not found
 if (!$found) {
  if (HomeWebApp::DEBUG) {
   echo "category doesn't exist, in production, 404 is invoked";
   exit;
  } else {
   $this->_forward("error","Error",null,
    array("error_handler"=>Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_ACTION));
   return;
  }
 }

 //right block
 $this->righBlock();
 
 $this->view->category=$category;

 //page title
 $this->view->pageTitle=$this->view->trans("Articles in the category")." "
  .$this->view->escape($category->getName());

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogCategoryName($category->getTitleUrl());

 $allArticleCount=$category->getArticleCount();

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
 
 //page navigator
 $pgNav=new PageNav($pageNo,$allArticleCount,Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=$urlEnum->blogCategory;

 //category articles in the current page
 $this->view->articles=$category->getArticles($pageNo,$pgNav->getPerPageItems());

 $this->view->allArticleCount=$allArticleCount;
 
 //page navigator
 $this->view->pageNav=$pgNav;
}

//==============================================================================
/**
 * Builds and prints RSS feed for some count if recent articles.
 * Sets "isFeed" parameter, which says to controller plugin that this
 * response isn't part of the normal site template.
 */
public function feedAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 // Disable layout
 $this->_helper->layout->disableLayout();
 $article = new Article();
 $recentArticles=$article->getAll(1,10);

 $lastUpdate=current($recentArticles)->getPosted();
 $urlEnum=AppUrlEnum::getInstance();

 $baseUri=AppBaseUrl::getUri();

 //prepare an array for a feed
 $feedArray = array(
  "title" => "Mirin's blog",
	"link" => $baseUri.$urlEnum->blogRSSfeed,
	"lastUpdate" => strtotime($lastUpdate),
  "charset" => "utf-8",
	"description" => "Blog about coding, and other stuff",
	"author" => "Mirek Kubelik",
	"email" => "koubel@volny.cz",
	"copyright" => "Koubel, all rights reserved",
  "generator" => 'Zend Framework Zend_Feed',
	"language" => "cs",
			'entries' => array()
		);

 foreach ($recentArticles as $article) {
  //get article text in the html 
  $this->view->article=$article;
  $articleHtml=$this->view->render("pureHtml.phtml");
  unset($this->view->article);
 
  $urlEnum->setBlogEntryName($article->getTitleUrl());
  $articleUrlUnique=$baseUri.$urlEnum->blogEntry;

  //resolves local links like <a href="/homepage/blog">link</a> into
  //<a href="http://mysite/homepage/blog">link</a>
  $articleHtml=preg_replace("/\s(href|src)=\"\//"," $1=\"$baseUri/",$articleHtml);
 
  //resolves local links for the footnoteslike <a href="#fnt">link</a> into
  //<a href="http://mysite/homepage/blog#fnt">link</a>
  $articleHtml=preg_replace("/<a href=\"#/","<a href=\"$articleUrlUnique#",
   $articleHtml);

  $feedArray['entries'][] = array(
    "guid" => $articleUrlUnique,
    "title" => iconv("iso-8859-2","utf-8",$article->getTitle()),
	  "link" => $articleUrlUnique,
	  "description" => iconv("iso-8859-2","utf-8",
      strip_tags(Article::getFirstParagraph($articleHtml))),
	  "lastUpdate" => strtotime($article->getPosted()),
	  "content" => iconv("iso-8859-2","utf-8",Article::escapeCode($articleHtml))
	 );
 }
 
 $feed = Zend_Feed::importArray($feedArray, 'rss');
 $feed->send();

 //response is feed
 $this->_setParam("isFeed",true);
}

//==============================================================================
/**
 * Builds and print RSS feed for some recent comments.
 * Sets "isFeed" parameter, which says to controller plugin that this
 * response isn't part of the normal template.
 */
public function commentFeedAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 // Disable layout
 $this->_helper->layout->disableLayout();
 $comment=new Comment();
 //last 20 recent comments
 $recentComments=$comment->getAll(1,20);

 $lastUpdate=current($recentComments)->getPosted();
 $urlEnum=AppUrlEnum::getInstance();

 $baseUri=AppBaseUrl::getUri();

 //prepare an array for a feed
 $feedArray = array(
  "title" => "Mirin's blog - recent comments",
	"link" => $baseUri.$urlEnum->blogRSScommentFeed,
	"lastUpdate" => strtotime($lastUpdate),
  "charset" => "utf-8",
	"description" => "recent comments in Mirin's blog",
	"author" => "Mirek Kubelik",
	"email" => "koubel@volny.cz",
	"copyright" => "Koubel, all rights reserved",
  "generator" => 'Zend Framework Zend_Feed',
	"language" => "cs",
			'entries' => array()
		);

 foreach ($recentComments as $comment) {
  $relatedArticle=$comment->getArticle();
  //set article params in the url enumerator
  $urlEnum->setBlogEntryName($relatedArticle->getTitleUrl());
  //comment order in all article comments
  $order=$comment->getOrderforArticle();
  $message=iconv("iso-8859-2","utf-8",Comment::commentBody( $comment->getMessage() ));
  //unique comment URL, used for the guid and link
  $commentUrlUnique=$baseUri.$urlEnum->blogEntry."#comment$order";

  $feedArray['entries'][] = array(
   "guid" => $commentUrlUnique,
   "title" => iconv("iso-8859-2","utf-8",$comment->getAuthorName()
     . " ("
     . $relatedArticle->getTitle()
     . ")"
    ),
	 "link" => $commentUrlUnique,
	 "description" => $message,
	 "lastUpdate" => strtotime($comment->getPosted()),
	 "content" => $message
	 );
 }
 
 $feed = Zend_Feed::importArray($feedArray, 'rss');
 $feed->send();

 //response is feed
 $this->_setParam("isFeed",true);
}

//==============================================================================
/**
 * Views result from the main search form, it searches for an atricles
 * via simple regexp for article body. 
 */
public function searchResultAction()
{
 //right block
 $this->righBlock();
 //page title
 $this->view->pageTitle=$this->view->trans("Found articles");

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
 
 //search object
 $search=Search::factorySearch("SearchBlogArticle");
 //set the pagination
 $search->setPageParams($pageNo,Article::ARTICLES_PERPAGE);
 
 $params = Utils::getHTTPParamsFiltered(array("submit"));
 
 //set search parameters if form is submitted, otherwise these are set in the
 //factory method
 if (isset($params["searchText"]))
  $search->setParams($params);

 $article=new Article();

 //found items
 $this->view->foundItems=$article->getFound($search);

 //get found items count, if no items found sets count in the view and exits
 if (! $itemsCount=$search->itemsCount() ) {
  $this->view->allFoundItemsCount=$itemsCount;
  return;
 } 
 
 //save search params to session
 $search->saveToSession();

 //page navigator
 $pgNav=new PageNav($pageNo,$itemsCount,Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=AppUrlEnum::getInstance()->blogSearchResult;

 $this->view->allFoundItemsCount=$itemsCount;
 
 //page navigator
 $this->view->pageNav=$pgNav;
}

//==============================================================================
/**
 * Actions for a view of articles for one month.
 */
public function viewArchiveAction()
{
 //right block
 $this->righBlock();

 $year=$this->_getParam("year");
 $month=$this->_getParam("month");
 $day=$this->_getParam("day");

 $app=HomeWebApp::getInstance();

 $date=new Zend_Date(array('year' => $year,
    'month' => $month,
    'day'=>is_null($day) ? 1 : $day,
    $app->getLocale())
  );

 $this->view->month=iconv("utf-8","iso-8859-2",
  Utils::getMonthFirstCaseName($month,$app->getLocale())
 );
 $this->view->year=$date->get(Zend_Date::YEAR);

 //page title
 $this->view->pageTitle=$this->view->trans("Articles archive for")." ";
 if (!is_null($day))
  $this->view->pageTitle.=$date->get(Zend_Date::DATES);
 else
  $this->view->pageTitle.=$this->view->month." ".$this->view->year;

 
 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogArchiveYear($year);
 $urlEnum->setBlogArchiveMonth($month);
 $urlEnum->setBlogArchiveDay($day);

 $article=new Article();
 //all articles for the current archive
 $allArticleCount=$article->getArchiveItemsCount($year,$month,$day);

 $this->view->allArticleCount=$allArticleCount;

 if ($allArticleCount==0)  {
  $this->view->articles=array();
  return;
 }

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;

 //page navigator
 $pgNav=new PageNav($pageNo,$allArticleCount,Article::ARTICLES_PERPAGE);
 $pgNav->MainUrl=$urlEnum->blogArchive;

 //articles in the current page for current archive year and month
 $this->view->articles=$article->getArchiveItems($year,$month,$day,$pageNo,$pgNav->getPerPageItems());

 //page navigator
 $this->view->pageNav=$pgNav;
}

//==============================================================================
/**
 * View archive.
 * Response is composed from two action view helpers, one article-calendar-archive
 * and second article-month-stat. Both are called from this view template.
 * This action is used for response of ajax calls.
 */
public function viewArchiveWidgetsAction()
{
 // Disable layout
 $this->_helper->layout->disableLayout();
 //set encode flag, because all ajax html response must be in utf-8
 $this->_setParam("encodeUTF8",true);
}

//==============================================================================
/** Sets cokie for the comment form data.
 * All data variables are saved in cookie as serialized array
 * @param string $username the username of a commentator
 * @param string $email the commentator email
 * @param string $www the commentator homepage
 */  
public static function setCommentCookie($username,$email,$www)
{
 $arr=array("username"=>$username,"mainEmail"=>$email,"www"=>$www); 

 /*
 $cookieDomain=AppBaseUrl::getCookieDomain();
 $this->getResponse()->addCookie(new Zend_Http_Cookie(
    self::COMMENTCOOKIE
   ,serialize($arr)
   ,!$cookieDomain ? HttpResponse::FALSE_DOMAIN : $cookieDomain
   ,strtotime("+1 year")
   ,AppUrlEnum::getInstance()->blogIndex
   ,false
   )
  );
 */
 
 setCookie(self::COMMENTCOOKIE,serialize($arr),strtotime("+1 year"),
  AppUrlEnum::getInstance()->blogIndex,
  AppBaseUrl::getCookieDomain()
 );
}

//==============================================================================
public function articleRatingAction()
{
 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 //this is ajax request with json response, hence disable layout
 $this->_helper->layout->disableLayout();

 $data=array(
   "articleId"=>$this->_getParam("articleId"),
   "rating"=>$this->_getParam("rating")
 );

 $filters = array(
  'articleId' => 'StringTrim',
  'rating' => 'StringTrim'
 );

 $validators = array(
  'articleId'   => array('Int', 'presence' => 'required'),
  'rating'   =>  array('Int', 'presence' => 'required')
 );

 $inputFilter = new Zend_Filter_Input($filters,$validators);
 $inputFilter->setData($data);

 $message="";

 if ($inputFilter->isValid()) {
  $articleId=$inputFilter->articleId;
  $rating=$inputFilter->rating;
  $remIP=$this->getRequest()->REMOTE_ADDR;
 
  $translator=HomeWebApp::getInstance()->getTranslate();
  
  $articleRating=new ArticleRating();

  try {
   $articleRating->setRating($rating);
   $articleRating->setArticleId($articleId);
   $articleRating->setIpaddr($remIP);

   if ($this->getRequest()->has($articleRating->getCookieName()) ||
       $articleRating->wasVoted() 
      ) 
   {
    $message=$translator->_("You already voted").".";
   } else {
    $articleRating->insert();

    //set cookie
    setCookie($articleRating->getCookieName(),1,strtotime("+1 month"),
     AppUrlEnum::getInstance()->blogIndex,
     AppBaseUrl::getCookieDomain()
    );

    $message=$translator->_("Thank you").".";
   }

  } catch (Zend_Db_Exception $e) {
   //$message="Cannot save record, database error: ".$e->getMessage();
   $message="database error".$e->getMessage();
  }
 } else {
  $message="invalid input data";
 }

 $retArr=$articleRating->getRatingInfo();
 $retArr["message"]=$message;
 //adjust average
 $retArr["average"]=sprintf("%.2f",$retArr["average"]);

 echo Zend_Json::encode($retArr);
}

//==============================================================================
/** The action for an article preview.
 * This action is invoked from ajax call, so we need to disable a layout.
 */
public function articlePreviewAction()
{
 $config = Zend_Registry::get("conf");

 //disable layouts
 $this->_helper->layout->disableLayout();

 //temporary article object
 $article=new Article();
 //set properties from the request values
 $article->setFormat($this->_getParam("formatType"));
 $article->setText($this->_getParam("articleText"));

 $this->view->article=$article;
}

//==============================================================================
/** The action for an comment preview.
 * This action is invoked from ajax call, so we need to disable a layout.
 */
public function commentPreviewAction()
{
 $config = Zend_Registry::get("conf");

 //disable layouts
 $this->_helper->layout->disableLayout();

 $commentText=Comment::prepareCommentText($this->_getParam("message"));

 $this->view->commentText=$commentText;
}

//==============================================================================
/**
 * Renders the list of links for the months and article count for every month
 * in the list. Is called from the Zend_View_Helper action as the widget.
 */
public function articleMonthStatAction()
{
 $monthArchive=$this->_getMonthArchive();

 //remove current month statistic, because
 //this month stat is in the calendar archive head
 $monthStats=$monthArchive->getMonthStats();
 foreach ($monthStats as $key=>$monthSt) {
  if ($monthSt->getMonth()==$monthArchive->getMonth()
   && $monthSt->getYear()==$monthArchive->getYear()) {
   unset($monthStats[$key]);
  }
 }

 $this->view->stat=$monthStats;
}

//==============================================================================
/**
 * Renders the calendar for the article archive.
 * Is called from the Zend_View_Helper action as the widget.
 */
public function articleCalendarArchiveAction()
{
 $monthArchive=$this->_getMonthArchive();
 
 $this->view->articleArchive=$monthArchive;
}

//==============================================================================
/**
 * Creates month archive object and sets its properties from requests values
 * if they are available in the request.
 * @return ArticleMonthArchive
 */
private function _getMonthArchive()
{
 $year=$this->_hasParam("year") ? $this->_getParam("year") : date('Y');
 $month=$this->_hasParam("month") ? $this->_getParam("month") : date('n');

 $monthArchive=new ArticleMonthArchive();
 $monthArchive->setMonth($month);
 $monthArchive->setYear($year);

 return $monthArchive;
}

//==============================================================================
/**
 * Returs request parameters which are related to archive.
 * These parameters are: month, year, day
 * @return array
 */
public static function getArchiveParams()
{
 $archiveParams = array();
 $request=Zend_Controller_Front::getInstance()->getRequest();

 $paramsKeys=array("year","month","day");
 foreach($paramsKeys as $paramKey) {
  if ($param=$request->getUserParam($paramKey)) $archiveParams[$paramKey]=$param;
 }

 return $archiveParams;
}

//==============================================================================
/**
 * Handles the trackbacks.
 * Doesn't use viewRenderer and exits after the XML response message is sent.
 * Saves trackback into the database. TrackBack is checked if is spam or not.
 * @return string trackback response XML message - see {@link TrackBack::makeResponse()}
 */
public function trackbackAction()
{
 $article = new Article();
 $found=$article->findById($this->_getParam("id"));

 //entry for given id wasn't find
 if (!$found) {
  echo TrackBack::getResponse(false,"unknown entry id");
  exit;
 }

 if ($_SERVER["REQUEST_METHOD"]=="GET") {
  $this->_setParam("entry",$article->getTitleUrl());
  $this->_forward("view-entry");
  return;
 }

 //Don't auto render
 $this->_helper->viewRenderer->setNoRender();

 $filters = array(
    'url' =>'StringTrim',
    'title' => 'StringTrim',
    'blog_name' => 'StringTrim',
    'excerpt' => 'StringTrim'
 );

 $validators = array(
    'id' => array('Digits','presence' => 'required'),
    'url'   => array(new UrlValid(),'presence' => 'required'),
    'title' => array(),
    'blog_name' => array(),
    'excerpt' => array()
 );

 $inputFilter = new Zend_Filter_Input($filters, $validators);
 //sets a default escaper into the stripTags filter
 $inputFilter->setDefaultEscapeFilter(new Zend_Filter_StripTags());

 $data=TrackBack::prepareRequestData("iso-8859-2");
 $data["id"]=$this->_getParam("id");
 $inputFilter->setData($data);

 if (!$inputFilter->isValid()) {
  $message = "Data are invalid:";
  $message .= implode("" , Utils::getMessagesOneArray($inputFilter));
  echo TrackBack::getResponse(false,$message);
  exit;
 }

 //get full article Url
 $urlEnum=AppUrlEnum::getInstance();
 $baseUri=AppBaseUrl::getUri();
 $urlEnum->setBlogEntryName($article->getTitleUrl());
 $articleUrlUnique=$baseUri.$urlEnum->blogEntry;
 
 //url of the trackbacker
 $sourceUrl=$inputFilter->url;

 //trackback Api object
 $trackBackApi=new TrackBack($sourceUrl);
 
 //trackback domain object
 $trackBack=new TrackBackEntry();
 $trackBack->setArticleId($inputFilter->id);
 $trackBack->setIpAddr($this->getRequest()->REMOTE_ADDR);
 $trackBack->setUrl(substr($inputFilter->url,0,500));
 $trackBack->setTitle(substr($inputFilter->title,0,50));
 $trackBack->setBlogName(substr($inputFilter->blog_name,0,100));
 $excerpt=substr($inputFilter->excerpt,0,200);
 //remove dots (dots are added in some trackbacks excerpts from the wordpress blogs
 $excerpt=trim($excerpt,".");
 $trackBack->setExcerpt($excerpt);

 if ($trackBack->isExist()) {
  $trackBackApi->setErrorMessage("trackback entry with this url already exist");
  echo $trackBackApi->makeResponse(false);
  exit;
 }

 if (TrackBack::isTrackBackSpam($articleUrlUnique,$sourceUrl)) {
  $trackBack->setSpam();
  $trackBack->insert();
  $trackBackApi->setErrorMessage("entry url isn't found in the trackbacker page");
  echo $trackBackApi->makeResponse(false);
  exit;
 }

 //all OK save trackback and send successfull trackback response
 $trackBack->setSpam(false);
 $trackBack->insert();
 echo $trackBackApi->makeResponse(true);
 exit;
}

//==============================================================================
/**
 * Gets RDF for article trackback autodiscovery
 * @return string
 */
public function trackbackArticleRDF(Article $article)
{
 //get full article Url
 $urlEnum=AppUrlEnum::getInstance();
 $baseUri=AppBaseUrl::getUri();
 $urlEnum->setBlogEntryName($article->getTitleUrl());
 $urlEnum->setBlogEntryId($article->getId());

 $trackBackApi=new TrackBack(null,"iso-8859-2");
 $trackBackApi->setPingUrl($baseUri.$urlEnum->trackBackEntry);

 $datePosted=new Zend_Date($article->getPosted(),Zend_Date::ISO_8601,"en_US");

 return $trackBackApi->getRDfAutodiscover($datePosted->get(Zend_Date::RFC_822)
  ,$baseUri.$urlEnum->blogEntry
  ,substr(trim($article->getPlainText()),0,100)
  ,$article->getTitle()
  ,"Mirin"
 );
}

#END CLASS
}
