<?php
/**
 * CategoryArticle dao class for the M:N relation between categories and articles
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * CategoryArticle dao class for the M:N relation between categories and articles
 * 
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */ 
class Dao_CategoryArticle extends Dao
{
 const TABLE_NAME = "categoryarticle";
 protected $_name = self::TABLE_NAME;
 protected $_primary = array('category_id', 'article_id');

 protected $_referenceMap    = array(
  'Category' => array(
            'columns'           => array('category_id'),
            'refTableClass'     => 'Dao_Category',
            'refColumns'        => array('id')
        ),
  'Article' => array(
            'columns'           => array('article_id'),
            'refTableClass'     => 'Dao_Article',
            'refColumns'        => array('id')
        )
    );

//==============================================================================
/**
 * Gets count of articles for a specific category
 * @param int $categoryId the category identifier, which determines category
 * for which we want article count
 * @return int
 */
public function getArticleCount($categoryId)
{
 $sql="select count(*) from $this->_name where category_id=$categoryId";
 return $this->getAdapter()->fetchOne($sql);
}

//==============================================================================
/**
 * Gets articles for the particular category
 * @param int $categoryId the category identifier, which determines category
 * @param int $pageNo current page number, first page has number 1
 * @param int $perPageItems number of items per page
 * @return array
 */
public function getArticlesByCategory($categoryId,$pageNo=null,$perPageItems=null)
{
 $commentT=Dao_Comment::TABLE_NAME;
 $articleT=Dao_Article::TABLE_NAME;
 $authorT=Dao_Author::TABLE_NAME;

 $select=$this->getAdapter()->select()
   ->from($this->_name)
   ->from($articleT)
   ->joinLeft($commentT,"$commentT.article_id = $articleT.id",array("commentCount"=>"count($commentT.id)"))
   ->joinLeft($authorT,"$authorT.id = $articleT.author_id",array("authorName"=>"username"))
   ->where("$this->_name.article_id=$articleT.id")
   ->where("$this->_name.category_id=$categoryId")
   ->group("$articleT.id")
   ->order("$articleT.posted desc");

 self::$_dataSource->addSQLlimit($select,$pageNo,$perPageItems);

 return $select->query()->fetchAll();
}

#END CLASS
}
