<?php
/**
 * the model for the trackback
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Trackback model class
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class TrackBackEntry extends Model
{
 /**
  * database id for an trackback item
  * @var int 
  */
 private $_id;

 private $_articleId;

 private $_title;

 private $_excerpt;

 private $_url;

 private $_blogName;

 private $_posted;

 private $_isSpam;
 
 private $_ipAddr;
 
//==============================================================================
protected function _setDefaultDao()
{
 $this->setDao(new Dao_TrackBack());
}

//==============================================================================
public function getArticleId()
{
 return $this->_articleId;
}

//==============================================================================
public function setArticleId($id)
{
 $this->_articleId=$id;
}

//==============================================================================
public function getTitle()
{
 return $this->_title;
}

//==============================================================================
public function setTitle($title)
{
 $this->_title=$title;
}

//==============================================================================
public function getUrl()
{
 return $this->_url;
}

//==============================================================================
public function setUrl($url)
{
 $this->_url=$url;
}

//==============================================================================
public function getExcerpt()
{
 return $this->_excerpt;
}

//==============================================================================
public function setExcerpt($excerpt)
{
 $this->_excerpt=$excerpt;
}

//==============================================================================
public function getBlogName()
{
 return $this->_blogName;
}

//==============================================================================
public function setBlogName($blogName)
{
 $this->_blogName=$blogName;
}

//==============================================================================
public function getPosted()
{
 return $this->_posted;
}

//==============================================================================
public function setPosted($posted)
{
 $this->_posted=$posted;
}
//==============================================================================
public function setSpam($status=true)
{
 $this->_isSpam=$status;
}

//==============================================================================
public function getIpAddr()
{
 return $this->_ipAddr;
}

//==============================================================================
public function setIpAddr($ipAddr)
{
 $this->_ipAddr=$ipAddr;
}

//==============================================================================
/**
 * Returns trackbacks for the particular article.
 * @param Article $article
 * @return array array of the TrackBackEntry objects
 * @todo use findDependentRowset with a new select functinality in the ZF 1.5
 */
public function getByArticle(Article $article)
{
 /*$currentArticle=$article->getDaoRow();
 $articleComments=$currentArticle->findDependentRowset("Dao_TrackBack");
 */

 return $this->_getSet($this->getDao()->getByArticleId($article->getId()));
}

//==============================================================================
public function getCountByArticle(Article $article)
{
 return $this->getDao()->getCountByArticle($article->getId());
}

//==============================================================================
public function findById($id)
{
 $row=$this->getDao()->find($id)->current();
 $this->_setFromDaoRow($row->toArray());
}

//==============================================================================
public function insert()
{
 $this->getDao()->insert(
     array("article_id"=>$this->_articleId,
      "title"=> (empty($this->_title)? NULL : $this->_title),
      "excerpt"=> (empty($this->_excerpt)? NULL : $this->_excerpt),
      "url"=> $this->_url,
      "blogName"=> (empty($this->_blogName)? NULL : $this->_blogName),
      "posted"=>new Zend_Db_Expr('NOW()'),
      "isSpam"=> $this->_isSpam ? 1 : 0,
      "ipAddr"=> $this->_ipAddr
     )
 );
}

//==============================================================================
protected function _setPropFromArray(&$arr)
{
 foreach($arr as $key=>$val) {
  $this->{"_".$key}=$val;
 }
}

//==============================================================================
public function isExist()
{
 return $this->getDao()->entryExist($this->_articleId,$this->_url);
}

//==============================================================================
protected function _inflectDaoRowIndexes(&$arr)
{
 $notAllowedIndexes=array("dao");

 $keys=array_keys($arr);
 foreach($keys as $key) {
  //unset not allowed indexes
  if (in_array($key,$notAllowedIndexes))
   throw new Exception("you cannot use $key index as the column index");

  switch ($key) {
   case "article_id":
    Utils::renameKey($arr,$key,"articleId");
    break;
   default:
    break;
  }
 }
}

#END CLASS
}
