<?php
/**
 * Implementation for the helper for one month archive link.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * The helper for one archive links. The link has the article count.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class HomeWebApp_View_Helper_MonthArchiveLink
{
 /**
  * locale
  * @var Zend_Locale
  */
 private $_locale;

 /**
  * url enumerator
  * @var AppUrlEnum
  */
 private $_urlEnum;

//==============================================================================
/** Sets the locale and enumerator from main application instances.
 */
public function __construct()
{
 $this->_locale = HomeWebApp::getInstance()->getLocale();
 $this->_urlEnum = AppUrlEnum::getInstance();

 //reset the archive day, necessary, because AppUrlEnum is singleton, it's
 //application persistenced
 $this->_urlEnum->setBlogArchiveDay(null);
}

//==============================================================================
/**
 * Renders the archive hyperlink for a month. 
 * @param ArticleMonthArchive $archive
 * @return string xhtml markup for the list
 */
public function monthArchiveLink(ArticleMonthArchive $archive)
{
 $xHtml="";
 
 $this->_urlEnum->setBlogArchiveYear($archive->getYear());
 $this->_urlEnum->setBlogArchiveMonth($archive->getMonth());

 $monthName = iconv("utf-8","iso-8859-2",Utils::getMonthFirstCaseName($archive->getMonth(),$this->_locale));

 $xHtml.= "<a href=\""
   . $this->_urlEnum->blogArchive
   . "\">"
   . $monthName
   . " "
   . $archive->getYear() . " "
   . "(" . $archive->getArticleCount() . ")"
   . "</a>";
  
 return $xHtml;
}

#END CLASS
}
