<?php
/**
 * An implementation of the pagination view helper.
 * 
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * View helper for the pagination navigator.
 *
 * Properties are set by {@link PageNav} object, which is used only for the
 * computing of pagination parameters. Supports translation
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class HomeWebApp_View_Helper_PageNavigator
{
 /**
  * name for the double-colon page number parametr in the $_urlWithParam
  */
 const PARAMNAME_PAGENUM="pageNum";
 
 /**
  * URL without "page navigation" parameters
  *
  * After this url the page navigation URL parametr will be added,
  * see {@link _buildUrl()}
  * @var string
  */
 private $_mainScriptUrl;

 /**
  * final url with the named (e.g. ":pageNum") part, which will be replaced by the
  * page number.
  * This is built through the {@link _buildUrl()} method.
  * @var string
  */
 private $_urlWithParam;

 /** how many pages as url links can be printed in page number line
  * @var int
  */
 private $_maxPageRend=10;

 /**
  * current page number, it begins on 1
  * @var int
  */
 private $_currentPageNo;

 /**
  * all page count
  * @var int
  */
 private $_allPageCount;

 /**
  * final XHTML content for the page navigator, this is return value from the
  * helper.
  * @var string
  */
 private $_xhtml;

 /**
  * language translator
  * @var Zend_Translate
  */
 private $_translator;

//==============================================================================
public function pageNavigator(PageNav $pageNav)
{
 $this->_currentPageNo=$pageNav->getCurrentPageNo();
 $this->_allPageCount=$pageNav->getAllPageCount();
 $this->_mainScriptUrl=$pageNav->getMainScriptUrl();
 $this->_translator=HomeWebApp::getInstance()->getTranslate();
 $this->_buildUrl();

 if ($this->_allPageCount>1) {
  //firts and previous part
  if ($this->_currentPageNo==1) {
   $this->_xhtml.="|&lt; &lt; ";
  } else {
   $this->_xhtml.="<a href=\"".$this->_makeUrl(1)."\" title=\""
    . $this->_translator->_("First page")."\">|&lt;</a> "
    . "<a href=\"".$this->_makeUrl($this->_currentPageNo-1)."\" title=\""
    . $this->_translator->_("Previous page")."\">&lt;</a> ";
  }

  //page numbers
  $this->_renderNmbLine();

  //next and last part
  if ($this->_currentPageNo==$this->_allPageCount) {
   $this->_xhtml.="&gt; &gt;|";
  } else {
   $this->_xhtml.="<a href=\"".$this->_makeUrl($this->_currentPageNo+1)."\" title=\""
    . $this->_translator->_("Next page")."\">&gt;</a> "
    . "<a href=\"".$this->_makeUrl($this->_allPageCount)."\" title=\""
    . $this->_translator->_("Last Page")."\">&gt;|</a> ";
  }
  return iconv("utf-8","iso-8859-2",$this->_xhtml);
 }
 return "";
}

//==============================================================================
/**
 * Prepares url, makes final url template with double colon parameters.
 * Fills self::_urlWithParam.
 */
private function _buildUrl()
{
 if ($pos=strpos($this->_mainScriptUrl,"?") !== false) {
  $this->_urlWithParam=$this->_mainScriptUrl."&";
 } else {
  $this->_urlWithParam=$this->_mainScriptUrl."?";
 }

 $this->_urlWithParam.=PageNav::NAVPARAM
  ."=".self::PARAMNAME_PAGENUM;
}

//==============================================================================
/**
 * Replaces parameters in the final url and returns complete  url.
 * @return string
 */
private function _makeUrl($pageNum)
{
 return str_replace(self::PARAMNAME_PAGENUM,$pageNum,$this->_urlWithParam);
}

//==============================================================================
/**
 * Renders the line with page numbers to the {@link $_xhtml}.
 */
private function _renderNmbLine()
{
 if ($this->_currentPageNo>$this->_maxPageRend) {
  $startPn=$this->_currentPageNo-4;
 } else {
  $startPn=1;
 }

 $endPn=$startPn+$this->_maxPageRend;

 for($i=$startPn;$i<$endPn;$i++) {
  //page count reached, stop rendering and return
  if ($i>$this->_allPageCount) return;
  if ($i==$this->_currentPageNo) {
   $this->_xhtml.="<span style=\"font-weight: bold\">$i</span> \n";
   continue;
  }
  $url=$this->_makeurl($i);
  $this->_xhtml.="<a href=\"$url\" title=\""
   . $this->_translator->_("Page no.")." $i\">$i</a> \n";
 }
}

#END CLASS
}
